/*
  Bart Venneker December 2020
  Countdown to the new year in seconds..
  This code is in the public domain.

*/
#define STASSID "BlackPearl"
#define STAPSK  "aa1bb2cc3D"


#include <ESP8266WiFi.h>
#include <WiFiUdp.h>

#define NJ2020 1577833199UL
#define NJ2021 1609455599UL  
#define NJ2022 1640991599UL
#define NJ2023 1672527599UL
#define NJ2024 1704063599UL
#define NJ2025 1735685999UL
#define NJ2026 1767221999UL
#define NJ2027 1798757999UL
#define NJ2028 1830293999UL
#define NJ2029 1861916399UL
#define NJ2030 1893452399UL
#define NJ2031 1924988399UL

#define MAX7219_DIN D6
#define MAX7219_CS  D4
#define MAX7219_CLK D5

#define enableLM2596 D7
    
const char * ssid = STASSID; // your network SSID (name)
const char * pass = STAPSK;  // your network password

unsigned long Countdown = 0;

unsigned int localPort = 2390;      // local port to listen for UDP packets
bool firstUpdate=false;

/* Don't hardwire the IP address or we won't get the benefits of the pool.
    Lookup the IP address for the host name instead */
IPAddress timeServerIP;
const char* ntpServerName = "nl.pool.ntp.org";

const int NTP_PACKET_SIZE = 48; // NTP time stamp is in the first 48 bytes of the message

byte packetBuffer[ NTP_PACKET_SIZE]; //buffer to hold incoming and outgoing packets

// A UDP instance to let us send and receive packets over UDP
WiFiUDP udp;

void setup() {
  pinMode(enableLM2596,OUTPUT);  
  digitalWrite(enableLM2596,HIGH);
  
  digitalWrite(MAX7219_CS, HIGH);
  pinMode(MAX7219_DIN, OUTPUT);
  pinMode(MAX7219_CS, OUTPUT);
  pinMode(MAX7219_CLK, OUTPUT);
  output(0x0f, 0x00); //display test register - test mode off
  output(0x0c, 0x01); //shutdown register - normal operation
  output(0x0b, 0x07); //scan limit register - display digits 0 thru 7
  output(0x0a, 0x01); //brightness 0 to f
  output(0x09, 0xff); //decode mode register - CodeB decode all digits

  for (int y=1; y<9 ; y++) output(y, 0x0f);
  
  Serial.begin(115200);
  Serial.println();
  Serial.println();

  // We start by connecting to a WiFi network
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, pass);
  int c=8;
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);     
    output(c--, 0xff);
    if (c<0){
      c=8;
      for (int y=1; y<9 ; y++) output(y, 0x0f);
      }
  }
  output(8, 0xff);
  output(1, 0xff);
  Serial.println("");

  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  Serial.println("Starting UDP");
  udp.begin(localPort);
  Serial.print("Local port: ");
  Serial.println(udp.localPort());
}

void output(byte address, byte data) {

  digitalWrite(MAX7219_CS, LOW);
  shiftOut(MAX7219_DIN, MAX7219_CLK, MSBFIRST, address);
  shiftOut(MAX7219_DIN, MAX7219_CLK, MSBFIRST, data);
  digitalWrite(MAX7219_CS, HIGH);

}

int getNTP = 0;
bool ref = false;

void loop() {

  if (getNTP % 10 ==0){
    if (firstUpdate) updateDisplay(ref);
    getNTP = 0;
    WiFi.hostByName(ntpServerName, timeServerIP);
    sendNTPpacket(timeServerIP); // send an NTP packet to a time server
    // wait to see if a reply is available
    delay(1000);
    int cb = udp.parsePacket();
    
    if (Countdown > 0) Countdown -=1; 
    
    if (cb) {   
      ref=true;  
      firstUpdate=true;
      // We've received a packet, read the data from it
      udp.read(packetBuffer, NTP_PACKET_SIZE); // read the packet into the buffer
      unsigned long highWord = word(packetBuffer[40], packetBuffer[41]);
      unsigned long lowWord = word(packetBuffer[42], packetBuffer[43]);    
      unsigned long secsSince1900 = highWord << 16 | lowWord;

      // Unix time starts on Jan 1 1970. In seconds, that's 2208988800:
      const unsigned long seventyYears = 2208988800UL;
    
      // subtract seventy years to get the current unix time:
      unsigned long epoch = secsSince1900 - seventyYears -1;
            
      if      (epoch < NJ2020) Countdown = NJ2020 - epoch;
      else if (epoch < NJ2021) Countdown = NJ2021 - epoch;
      else if (epoch < NJ2022) Countdown = NJ2022 - epoch;
      else if (epoch < NJ2023) Countdown = NJ2023 - epoch;
      else if (epoch < NJ2024) Countdown = NJ2024 - epoch;
      else if (epoch < NJ2025) Countdown = NJ2025 - epoch;
      else if (epoch < NJ2026) Countdown = NJ2026 - epoch;
      else if (epoch < NJ2027) Countdown = NJ2027 - epoch;
      else if (epoch < NJ2028) Countdown = NJ2028 - epoch;
      else if (epoch < NJ2029) Countdown = NJ2029 - epoch;
      else if (epoch < NJ2030) Countdown = NJ2030 - epoch;
      else if (epoch < NJ2031) Countdown = NJ2031 - epoch;
    }    
  }
    if (firstUpdate)  updateDisplay(ref);
    
    delay(1000);
    if (Countdown > 0) {
      Countdown -=1; 
      getNTP +=1;
      ref=false;
    }else {
      getNTP =10;
      ref=true;  
      Countdown=31536000UL;
      }
    
    if (Countdown > 31017600UL or Countdown < 1){
        // de eerste week van het jaar is het ledbordje aan
        digitalWrite(enableLM2596,LOW);
    } else{
        digitalWrite(enableLM2596,HIGH);
    }
}

void updateDisplay(bool refreshed){
  unsigned x =0;
  
  output(0x0c, 0x00); // display off
  int z =0;
  for (int c=1 ; c<9 ; c++){
    x= (Countdown / (unsigned long) pow(10,c-1)) % 10;
    
    // Light the dot when data was refreshed
    if (refreshed and c==1) x=x + 0x0f0;
    
    output(c, x); 
    if (x>0) z=0;
    else z++;
    }

  // remove the leading zeros
  for (int c=8 ; c> 8-z ; c--){
    output(c, 0x0f);  
  }
  
  output(0x0c, 0x01); // display on
  
  
}
  
// send an NTP request to the time server at the given address
void sendNTPpacket(IPAddress& address) {
  // Serial.println("sending NTP packet...");
  // set all bytes in the buffer to 0
  memset(packetBuffer, 0, NTP_PACKET_SIZE);
  // Initialize values needed to form NTP request
  // (see URL above for details on the packets)
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12]  = 49;
  packetBuffer[13]  = 0x4E;
  packetBuffer[14]  = 49;
  packetBuffer[15]  = 52;

  // all NTP fields have been given values, now
  // you can send a packet requesting a timestamp:
  udp.beginPacket(address, 123); //NTP requests are to port 123
  udp.write(packetBuffer, NTP_PACKET_SIZE);
  udp.endPacket();
}
